#!/usr/bin/env node

var deg = require('./index');
var helper = require('./lib/helper');

var config = require(process.cwd() + '/package.json').config;
var dwJson = require(process.cwd() + '/dw.json');

const data = function(token) {
    if (config) {
        var { data } = config;
    } else {
        var data = {
            file_name: 'site-import',
            directories: [
                'site-import/'
            ]
        };
    }
    
    deg.archive('data', data.file_name, data.directories, token); 
}

const code = function(token) {
    var archiveName = dwJson.code_version || dwJson.version || 'version2';
    // console.log(`code version name: ${JSON.stringify(archiveName, null, 1)}`);

    var cartridges = helper.getCartridges();
    // console.log(`cartridge list: ${JSON.stringify(cartridges, null, 1)}`);

    helper.printIncludedCartridges(cartridges);
    
    deg.archive('code', archiveName, cartridges, token);
}

const migrate = function(action, args, token) {
    switch (action) {
        case 'create':
            deg.migration.create(args[0]);
            break;
        case 'set':
            deg.migration.set(args[0], token);
            break;
        case 'apply':
            deg.migration.apply(token);
            break;
        default:
            console.log('Command not recognized.');
            console.log('Available commands: add, set, apply');
    }

    return;
}

helper.getAuthToken().then((token) => {
    switch (process.argv[2]) {
        case 'data':
            data(token);
            break;
        
        case 'code':
            code(token);
            break;

        case 'migrate':
            migrate(process.argv[3], process.argv.slice(4), token);
            break;
        
        default:
            console.log('Command not recognized.');
            console.log('Available commands: data, code, migrate');
    }
}).catch((error) => {
    console.log('There was a problem authenticating. Make sure credentials are correct in dw.json');
});