var archiver = require('archiver');
var code = require('./code');
var data = require('./data');
var fs = require('fs');
var path = require('path');

module.exports = function(mode, fileName, directories, token) {
    return new Promise((resolve, reject) => {
        // remove backslashes and periods from archive names
        fileName = fileName.replace('/','-').replace(/\./g, '_');
        
        var output = fs.createWriteStream(process.cwd() + '/' + fileName + '.zip');    
        var archive = archiver('zip', {
            zlib: { level: 9 }
        }); 

        global.fileName = fileName;
        global.mode = mode;
            
        output.on('close', function() {        
            if(global.mode === 'data' || global.mode === 'migration'){            
                data(global.fileName, token).then(
                    (response) => {
                        resolve(response);
                    },
                    (error) => {
                        reject(error);
                    }
                ).catch((err) => {
                    reject(error);
                });            
            }
            else {            
                code(global.fileName, token);
            }
        });
        
        archive.on('error', function(err) {
            reject(err);
        });
        
        archive.pipe(output);        
        directories.forEach(function(dir){
            if(global.mode === 'data'){
                fs.readdir(dir, {withFileTypes: true}, function(err, files){
                    if(!err){
                        console.log('\nsite archive includes: \n')
                        files.forEach(function(file){
                            // refactor...
                            if(['catalogs', 'inventory-lists', 'pricebooks'].indexOf(file.name) < 0 || process.argv.includes('full') || global.mode === 'migration') {
                                if( file.name === 'libraries' && !process.argv.includes('lib') && !process.argv.includes('full') ){                                
                                    return;
                                }                            
                                console.log(file.name);
                                // what is being archived...
                                var filePath = process.cwd() + '/site-import/' + file.name;
                                if(file.isFile()){                                
                                    archive.file(fileName + '/' + file.name);
                                }
                                if(file.isDirectory()){                                
                                    archive.directory(filePath, fileName + '/' + file.name);
                                }
                            }
                        });                    
                        archive.finalize();
                    }
                });
            } else if (global.mode === 'migration') {
                archive.directory(dir, fileName);
                archive.finalize();
            } else {
                // dir is the directory being added to the archive (path relative to archive location)
                // sfra/cartridges/app_storefront_base
                // let dirPath = typeof dir === 'string' ? dir : dir.path;
                // let dirName = typeof dir === 'string' ?  : dir.name;
                archive.directory(dir.path, (fileName + '/' + dir.name));
            }
        });

        if(mode === 'code'){
            archive.finalize();
        }

        return fileName;
    });
}